# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/19_gbe.participants.ipynb (unless otherwise specified).

__all__ = []

# Cell
%load_ext autoreload
%autoreload 2
from .data_provider import GBEProvider

import pandas as pd