# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/21_validation_tasks.ipynb (unless otherwise specified).

__all__ = []

# Cell
%load_ext autoreload
%autoreload 2
from .data_provider import DataProvider
import json
import pandas as pd
import seaborn as sns