# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"DataProvider": "01_data_provider.ipynb",
         "get_efficiently": "01_data_provider.ipynb",
         "DataProvider.store_interim": "01_data_provider.ipynb",
         "DataProvider.load_interim": "01_data_provider.ipynb",
         "DataProvider.get_phone_codebook": "01_data_provider.ipynb",
         "DataProvider.determine_phone_b07": "01_data_provider.ipynb",
         "DataProvider.check_participant_id": "01_data_provider.ipynb",
         "DataProvider.test_check_participant_id": "01_data_provider.ipynb",
         "DataProvider.set_dtypes": "01_data_provider.ipynb",
         "DataProvider.get_phone_data": "01_data_provider.ipynb",
         "DataProvider.get_ba_codebook": "01_data_provider.ipynb",
         "DataProvider.get_ba_data": "01_data_provider.ipynb",
         "DataProvider.get_baseline_drinking_data": "01_data_provider.ipynb",
         "DataProvider.get_duplicate_mov_ids": "01_data_provider.ipynb",
         "DataProvider.get_mov_data": "01_data_provider.ipynb",
         "DataProvider.get_alcohol_per_drink": "01_data_provider.ipynb",
         "DataProvider.get_two_day_data": "01_data_provider.ipynb",
         "GBEProvider": "02_gbe.data_provider.ipynb",
         "GBEProvider.get_gbe_from_movisense": "02_gbe.data_provider.ipynb",
         "GBEProvider.define_initial_dataset": "02_gbe.data_provider.ipynb",
         "GBEProvider.define_baseline_sessions": "02_gbe.data_provider.ipynb",
         "GBEProvider.find_gbe_game_starting_time": "02_gbe.data_provider.ipynb",
         "GBEProvider.check_time_between_sessions": "02_gbe.data_provider.ipynb",
         "determine_ema_session_numbers": "02_gbe.data_provider.ipynb",
         "check_repeating_sessions": "02_gbe.data_provider.ipynb",
         "increase_session_numbers": "02_gbe.data_provider.ipynb",
         "filter_repeating_sessions": "02_gbe.data_provider.ipynb",
         "GBEProvider.fix_repeating_ema_sessions": "02_gbe.data_provider.ipynb",
         "GBEProvider.get_gbe_data": "02_gbe.data_provider.ipynb",
         "GBEProvider.get_initial_baseline": "02_gbe.data_provider.ipynb",
         "GBEProvider.get_replication_baseline": "02_gbe.data_provider.ipynb",
         "GBEProvider.decode_gbe_string": "02_gbe.data_provider.ipynb",
         "GBEProvider.decode_gbe_strings": "02_gbe.data_provider.ipynb",
         "WMDataProvider": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.decode_wm_strings": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.get_wm_trial_types": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.add_trial_types": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.get_wm_data": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.filter_old_app_sessions": "03_gbe.wm.data_provider.ipynb",
         "WMDataProvider.filter_level_two_failures": "03_gbe.wm.data_provider.ipynb",
         "get_max_levels": "04_gbe.wm.scoring.ipynb",
         "get_perc_correct": "04_gbe.wm.scoring.ipynb",
         "add_implied_data": "04_gbe.wm.scoring.ipynb",
         "get_implied_perc_correct": "04_gbe.wm.scoring.ipynb",
         "get_perc_correct_predicted_sep_r": "04_gbe.wm.scoring.ipynb",
         "get_perc_correct_predicted_sep": "04_gbe.wm.scoring.ipynb",
         "get_perc_correct_predicted_sep_trial_r": "04_gbe.wm.scoring.ipynb",
         "get_perc_correct_predicted_sep_trial": "04_gbe.wm.scoring.ipynb",
         "icc_description_dict": "05_stats.ipynb",
         "describe_iccs": "05_stats.ipynb",
         "retest_plot": "05_stats.ipynb",
         "jitter": "05_stats.ipynb",
         "analyze_reliability": "05_stats.ipynb",
         "get_psych_iccs": "05_stats.ipynb",
         "get_model_calculated_icc": "05_stats.ipynb",
         "var_comp_plot": "05_stats.ipynb",
         "get_icc_and_variance": "05_stats.ipynb",
         "get_scores": "05_stats.ipynb",
         "get_time_effect": "05_stats.ipynb",
         "plot_time_effect": "05_stats.ipynb",
         "plot_time_effect_area": "05_stats.ipynb",
         "decompose_within_between": "05_stats.ipynb",
         "SSTDataProvider": "07_gbe.sst.data_provider.ipynb",
         "SSTDataProvider.decode_sst_strings": "07_gbe.sst.data_provider.ipynb",
         "SSTDataProvider.get_sst_data": "07_gbe.sst.data_provider.ipynb",
         "RTTDataProvider": "08_gbe.rtt.data_provider.ipynb",
         "RTTDataProvider.decode_rtt_strings": "08_gbe.rtt.data_provider.ipynb",
         "RTTDataProvider.get_rtt_data": "08_gbe.rtt.data_provider.ipynb",
         "ISTDataProvider": "09_gbe.ist.data_provider.ipynb",
         "ISTDataProvider.get_optimal_turns": "09_gbe.ist.data_provider.ipynb",
         "ISTDataProvider.decode_ist_strings": "09_gbe.ist.data_provider.ipynb",
         "ISTDataProvider.get_ist_data": "09_gbe.ist.data_provider.ipynb",
         "ISTDataProvider.filter_old_app_sessions": "09_gbe.ist.data_provider.ipynb",
         "integration_methods_wrapper": "10_gbe.sst.scoring.ipynb",
         "verbruggen_quantile": "10_gbe.sst.scoring.ipynb",
         "integration_without_replacement": "10_gbe.sst.scoring.ipynb",
         "get_integration_without_replacement_ssrts": "10_gbe.sst.scoring.ipynb",
         "mean_method_ssrt": "10_gbe.sst.scoring.ipynb",
         "get_mean_method_ssrts": "10_gbe.sst.scoring.ipynb",
         "get_ssrt_predicted_sep_r": "10_gbe.sst.scoring.ipynb",
         "get_ssrt_sep": "10_gbe.sst.scoring.ipynb",
         "get_ssrt_predicted_joint_r": "10_gbe.sst.scoring.ipynb",
         "get_ssrt_predicted_joint": "10_gbe.sst.scoring.ipynb",
         "get_percentage_gamble": "11_gbe.rtt.scoring.ipynb",
         "get_perc_gamble_predicted_sep_r": "11_gbe.rtt.scoring.ipynb",
         "get_perc_gamble_predicted_sep": "11_gbe.rtt.scoring.ipynb",
         "get_perc_gamble_predicted_joint_r": "11_gbe.rtt.scoring.ipynb",
         "get_perc_gamble_predicted_joint": "11_gbe.rtt.scoring.ipynb",
         "get_average_oversampling": "12_gbe.ist.scoring.ipynb",
         "get_oversampling_predicted_sep_r": "12_gbe.ist.scoring.ipynb",
         "get_oversampling_sep": "12_gbe.ist.scoring.ipynb",
         "get_oversampling_predicted_joint_r": "12_gbe.ist.scoring.ipynb",
         "get_oversampling_predicted_joint": "12_gbe.ist.scoring.ipynb",
         "custom_params": "20_plotting.ipynb",
         "icc_comparison_plot": "20_plotting.ipynb",
         "get_intense_drinking_data": "24_gbe.rtt.intense_sampling.ipynb",
         "pearson_r": "25_gbe.ema_drinking_checkpoint.ipynb",
         "lmer": "25_gbe.ema_drinking_checkpoint.ipynb",
         "cor": "25_gbe.ema_drinking_checkpoint.ipynb"}

modules = ["data_provider.py",
           "gbe/data_provider.py",
           "gbe/wm/data_provider.py",
           "gbe/wm/scoring.py",
           "stats.py",
           "gbe/wm/reliability.py",
           "gbe/sst/data_provider.py",
           "gbe/rtt/data_provider.py",
           "gbe/ist/data_provider.py",
           "gbe/sst/scoring.py",
           "gbe/rtt/scoring.py",
           "gbe/ist/scoring.py",
           "gbe/rtt/reliability.py",
           "gbe/sst/reliability.py",
           "gbe/ist/reliability.py",
           "gbe/baseline_drinking.py",
           "gbe/participants.py",
           "plotting.py",
           "validation_task.py",
           "gbe/rtt/singular_fit_inspection.py",
           "gbe/baseline_drinking/separate.py",
           "gbe/intense_sampling.py",
           "gbe/ema_drinking.py"]

doc_url = "https://hgzech.github.io/trr265/"

git_url = "https://github.com/hgzech/trr265/tree/master/"

def custom_doc_links(name): return None
